//+------------------------------------------------------------------+
//|                                                   XmlParser2.mq5 |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//|                                                   yu-sha@ukr.net |
//+------------------------------------------------------------------+
#property copyright "2009, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#import "XmlParserDll.dll"
bool GetCode(string &xml,string &code);
#import
//+------------------------------------------------------------------+
//| CXmlAttribute class                                              |
//+------------------------------------------------------------------+
class CXmlAttribute
  {
public:
   string            Name;
   string            Value;
  };
//+------------------------------------------------------------------+
//| CXmlElement class                                                |
//+------------------------------------------------------------------+
class CXmlElement
  {
private:
   bool              ParseCode(string const s,int &p);
public:
   string            Name;
   CXmlElement      *Elements[];
   CXmlAttribute    *Attributes[];
   string            Text;

   void             ~CXmlElement();
   void              ClearAttributes();
   void              ClearElements();
   string            SetXml(string xml);
   string            GetXml();
  };

// ----------------------------------------------------------------
const int N=6;
//+------------------------------------------------------------------+
//| Custom StringReplace                                             |
//+------------------------------------------------------------------+
void StringReplace_(string &s,string const from,string const to) 
  {
   int p=0;
   while(p<StringLen(s))
      if(StringSubstr(s,p,1)==from) 
        {
         s=StringSubstr(s,0,p)+to+StringSubstr(s,p+1);
         p+=StringLen(to);
        }
   else
      p++;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string TextPreProcess(string const text) 
  {
   string s=text;
   StringReplace_(s,"&","&amp;");
   StringReplace_(s,"<","&lt;");
   StringReplace_(s,">","&gt;");
   StringReplace_(s,"\"","&quot;");
   StringReplace_(s,"'","&apos;");
   return(s);
  };
// CXmlElement

void CXmlElement::~CXmlElement() 
  {
   ClearAttributes();
   ClearElements();
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXmlElement::ClearAttributes() 
  {
   for(int i=ArraySize(Attributes)-1; i>=0; i--)
      delete Attributes[i];
   ArrayResize(Attributes,0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXmlElement::ClearElements() 
  {
   for(int i=ArraySize(Elements)-1; i>=0; i--)
      delete Elements[i];
   ArrayResize(Elements,0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CXmlElement::ParseCode(string const s,int &p) 
  {
   int c=int(StringToInteger(StringSubstr(s,p,N))); p=p+N;
   Name=StringSubstr(s,p,c);  p=p+c;
   string cmd=StringSubstr(s,p,1); p++;
   while((cmd=="A") || (cmd=="N")) 
     {
      if(cmd=="A") 
        {
         ArrayResize(Attributes,ArraySize(Attributes)+1);
         Attributes[ArraySize(Attributes)-1]=new CXmlAttribute;
         c=int(StringToInteger(StringSubstr(s,p,N))); p=p+N;
         Attributes[ArraySize(Attributes)-1].Name=StringSubstr(s,p,c);  p=p+c;
         c=int(StringToInteger(StringSubstr(s,p,N))); p=p+N;
         Attributes[ArraySize(Attributes)-1].Value=StringSubstr(s,p,c);  p=p+c;
        }
      else
      if(cmd=="N") 
        {
         ArrayResize(Elements,ArraySize(Elements)+1);
         Elements[ArraySize(Elements)-1]=new CXmlElement;
         if(!Elements[ArraySize(Elements)-1].ParseCode(s,p))
            return(false);
        }
      cmd=StringSubstr(s,p,1); p++;
     }
   if(cmd=="T") 
     {
      c=int(StringToInteger(StringSubstr(s,p,N))); p=p+N;
      Text=StringSubstr(s,p,c);  p=p+c;
      cmd=StringSubstr(s,p,1); p++;
     };
   return(cmd=="D");
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string CXmlElement::SetXml(string xml) 
  {
   ClearAttributes();
   ClearElements();
   Text=NULL;
   string s,err;
   StringInit(s,10*StringLen(xml));
   GetCode(xml,s);
   if(StringSubstr(s,0,6)!="Error:") 
     {
      int p=1;
      ParseCode(s,p);
      return("");
     }
   else
      return(s);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string CXmlElement::GetXml() 
  {
   string s;
   s="<"+Name;
   for(int i=0; i<ArraySize(Attributes); i++)
      StringAdd(s," "+Attributes[i].Name+"=\""+TextPreProcess(Attributes[i].Value)+"\"");
   if((ArraySize(Elements)==0) && (Text==NULL))
      StringAdd(s,"/>");
   else 
     {
      StringAdd(s,">");
      for(int i=0; i<ArraySize(Elements); i++)
         StringAdd(s,Elements[i].GetXml());
      StringAdd(s,TextPreProcess(Text));
      StringAdd(s,"</"+Name+">");
     }
   return(s);
  };
//+------------------------------------------------------------------+
